@@ -4,6 +4,8 @@ Type descriptions are collections of typedesc instances.
 
 from __future__ import print_function
 from __future__ import unicode_literals
+import collections
+from collections.abc import Iterable
 from functools import cmp_to_key
 import textwrap
 from io import StringIO
@@ -13,6 +15,7 @@ import sys
 
 from ctypeslib.codegen import clangparser
 from ctypeslib.codegen import typedesc
+from ctypeslib.codegen import util
 
 import logging
 
@@ -43,7 +46,7 @@ class Generator(object):
             self.searched_dlls = searched_dlls
 
         self.done = set()  # type descriptions that have been generated
-        self.names = set()  # names that have been generated
+        self.names = list()  # names that have been generated
         self.macros = 0
         self.cross_arch_code_generation = cross_arch
 
@@ -150,7 +153,7 @@ class Generator(object):
             return self.type_name(t.typ, generate)
         elif isinstance(t, typedesc.PointerType):
             pointer_class = self.enable_pointer_type()
-            if t.typ.name == "c_char":
+            if t.typ.name in ["c_ubyte", "c_char"]:
                 self.enable_string_cast()
             return "%s(%s)" % (pointer_class, self.type_name(t.typ, generate))
         elif isinstance(t, typedesc.FunctionType):
@@ -199,40 +202,52 @@ class Generator(object):
         if self.generate_comments:
             self.print_comment(macro)
 
+        # get tokens types all the way to here ?
+        # 1. clang makes the decision on type casting and validity of data.
+        # let's not try to be clever.
+        # only ignore, undefined references, macro functions...
+        # 2. or get a flag in macro that tells us if something contains undefinedIdentifier /is not codegenable ?
+        # codegen should decide what codegen can do.
         if macro.args:
             print("# def %s%s:  # macro" % (macro.name, macro.args), file=self.stream)
             print("#    return %s  " % macro.body, file=self.stream)
-        elif isinstance(macro.body, list):
-            # we can't handle that
-            print("# %s = %s # macro" % (macro.name, ' '.join([str(_) for _ in macro.body])), file=self.stream)
-        elif isinstance(macro.body, typedesc.UndefinedIdentifier):
-            # just comment is out
-            print("# %s = %s # macro" % (macro.name, macro.body.name), file=self.stream)
-        elif isinstance(macro.body, str) or isinstance(macro.body, bool):
-            # either it's just a thing we gonna print, or we need to have a registered item
+        elif util.contains_undefined_identifier(macro):
+            # we can't handle that, we comment it out
+            if isinstance(macro.body, typedesc.UndefinedIdentifier):
+                print("# %s = %s # macro" % (macro.name, macro.body.name), file=self.stream)
+            else:  # we assume it's a list
+                print("# %s = %s # macro" % (macro.name, ' '.join([str(_) for _ in macro.body])), file=self.stream)
+        elif isinstance(macro.body, bool):
             print("%s = %s # macro" % (macro.name, macro.body), file=self.stream)
+            self.macros += 1
+            self.names.append(macro.name)
+        elif isinstance(macro.body, str):
+            body = macro.body
+            float_value = util.from_c_float_literal(body)
+            if float_value is not None:
+                body = float_value
+            # what about integers you ask ? body token that represents token are Integer here.
+            # either it's just a thing we gonna print, or we need to have a registered item
+            print("%s = %s # macro" % (macro.name, body), file=self.stream)
+            self.macros += 1
+            self.names.append(macro.name)
+        # This is why we need to have token types all the way here.
+        # but at the same time, clang does not type tokens. So we might as well guess them here too
+        elif util.body_is_all_string_tokens(macro.body):
+            print("%s = %s # macro" % (macro.name, ' '.join([str(_) for _ in macro.body])), file=self.stream)
+            self.macros += 1
+            self.names.append(macro.name)
         else:
-            # we just comment out all macro definitions, because values will be replace in situ
-            print("# %s = %s # macro" % (macro.name, macro.body), file=self.stream)
-        self.macros += 1
-        self.names.add(macro.name)
+            # this might be a token list of float literal
+            body = macro.body
+            float_value = util.from_c_float_literal(body)
+            if float_value is not None:
+                body = float_value
+            # or anything else that might be a valid python literal...
+            print("%s = %s # macro" % (macro.name, body), file=self.stream)
+            self.macros += 1
+            self.names.append(macro.name)
         return
-        # We don't know if we can generate valid, error free Python
-        # code. All we can do is to try to compile the code.  If the
-        # compile fails, we know it cannot work, so we comment out the
-        # generated code; the user may be able to fix it manually.
-        #
-        # If the compilation succeeds, it may still fail at runtime
-        # when the macro is called.
-        # mcode = "def %s%s: return %s # macro" % (macro.name, macro.args,
-        # macro.body)
-        # try:
-        #    compile(mcode, "<string>", "exec")
-        # except SyntaxError:
-        #    print >> self.stream, "#", mcode
-        # else:
-        #    print >> self.stream, mcode, '# Macro'
-        #    self.names.add(macro.name)
 
     _typedefs = 0
 
@@ -254,7 +269,7 @@ class Generator(object):
                 tp.name in sized_types):
             print("%s = ctypes.%s" % \
                   (name, sized_types[tp.name]), file=self.stream)
-            self.names.add(tp.name)
+            self.names.append(tp.name)
             return
         if tp.typ not in self.done:
             # generate only declaration code for records ?
@@ -272,9 +287,9 @@ class Generator(object):
             if isinstance(tp.typ, typedesc.Enumeration):
                 print("%s__enumvalues = %s__enumvalues" % \
                       (name, self.type_name(tp.typ)), file=self.stream)
-                self.names.add("%s__enumvalues" % name)
+                self.names.append("%s__enumvalues" % name)
 
-        self.names.add(tp.name)
+        self.names.append(tp.name)
         self._typedefs += 1
         return
 
@@ -347,7 +362,7 @@ class Generator(object):
                                                   self.type_name(tp.typ),
                                                   libname,
                                                   tp.name), file=self.stream)
-            self.names.add(tp.name)
+            self.names.append(tp.name)
             # wtypes.h contains IID_IProcessInitControl, for example
             return
 
@@ -365,68 +380,56 @@ class Generator(object):
                                           self.type_name(
                                               tp.init),
                                           [x for x in tp.typ.iterArgNames()]), file=self.stream)
-        else:
-            init_value = tp.init
-            if isinstance(tp.typ, typedesc.PointerType) or isinstance(tp.typ, typedesc.ArrayType):
-                if (isinstance(tp.typ.typ, typedesc.FundamentalType) and
-                        (tp.typ.typ.name in ["c_char", "c_wchar"])):
-                    # string
-                    # FIXME a char * is not a python string.
-                    # we should output a cstring() construct.
-                    init_value = repr(tp.init)
-                elif (isinstance(tp.typ.typ, typedesc.FundamentalType) and
-                      ('int' in tp.typ.typ.name or 'long' in tp.typ.typ.name)):
-                    # array of number
-                    # CARE: size of elements must match size of array
-                    # init_value = repr(tp.init)
-                    init_value = '[%s]' % ','.join([str(x) for x in tp.init])
-                    # we do NOT want Variable to be described as ctypes object
-                    # when we can have a python abstraction for them.
-                    # init_value_type = self.type_name(tp.typ, False)
-                    # init_value = "(%s)(%s)"%(init_value_type,init_value)
-                elif isinstance(tp.typ.typ, typedesc.Structure):
-                    self._generate(tp.typ.typ)
-                    init_value = self.type_name(tp.typ, False) + "()"
-                else:
-                    init_value = tp.init if tp.init is not None else (
-                                                                         self.type_name(tp.typ, False)
-                                                                     ) + "()"
-            elif isinstance(tp.typ, typedesc.Structure):
-                init_value = self.type_name(tp.typ, False)
-            elif isinstance(tp.typ, typedesc.FundamentalType) and tp.typ.name in ["c_char", "c_wchar"]:
-                if tp.init is not None:
-                    init_value = repr(tp.init)
-                else:
-                    init_value = '\'\\x00\''
+            self.names.append(tp.name)
+            return
+        elif isinstance(tp.typ, typedesc.PointerType) or isinstance(tp.typ, typedesc.ArrayType):
+            if (isinstance(tp.typ.typ, typedesc.FundamentalType) and
+                    (tp.typ.typ.name in ["c_ubyte", "c_char", "c_wchar"])):
+                # string
+                # FIXME a char * is not a python string.
+                # we should output a cstring() construct.
+                init_value = repr(tp.init)
+            elif (isinstance(tp.typ.typ, typedesc.FundamentalType) and
+                  ('int' in tp.typ.typ.name or 'long' in tp.typ.typ.name)):
+                # array of number
+                # CARE: size of elements must match size of array
+                # init_value = repr(tp.init)
+                init_value = '[%s]' % ','.join([str(x) for x in tp.init])
+                # we do NOT want Variable to be described as ctypes object
+                # when we can have a python abstraction for them.
+                # init_value_type = self.type_name(tp.typ, False)
+                # init_value = "(%s)(%s)"%(init_value_type,init_value)
+            elif isinstance(tp.typ.typ, typedesc.Structure):
+                self._generate(tp.typ.typ)
+                init_value = self.type_name(tp.typ, False) + "()"
             else:
-                # we want to have FundamentalType variable use the actual
-                # type default, and not be a python ctypes object
-                # if init_value is None:
-                #    init_value = ''; # use default ctypes object constructor
-                # init_value = "%s(%s)"%(self.type_name(tp.typ, False), init_value)
-                if tp.init is not None:
-                    init_value = tp.init
-                elif tp.typ.name in ['c_float', 'c_double', 'c_longdouble']:
-                    init_value = 0.0
-                else:
-                    # integers
-                    init_value = 0
-            #
-            # print it out
-            print("%s = %s # Variable %s" % (tp.name,
-                                             init_value,
-                                             self.type_name(tp.typ, False)), file=self.stream)
+                init_value = tp.init if tp.init is not None else (self.type_name(tp.typ, False)) + "()"
+        elif isinstance(tp.typ, typedesc.Structure):
+            init_value = self.type_name(tp.typ, False)
+        elif isinstance(tp.typ, typedesc.FundamentalType) and tp.typ.name in ["c_ubyte", "c_char", "c_wchar"]:
+            if tp.init is not None:
+                init_value = repr(tp.init)
+            else:
+                init_value = '\'\\x00\''
+        else:
+            # we want to have FundamentalType variable use the actual
+            # type default, and not be a python ctypes object
+            # if init_value is None:
+            #    init_value = ''; # use default ctypes object constructor
+            # init_value = "%s(%s)"%(self.type_name(tp.typ, False), init_value)
+            if tp.init is not None:
+                init_value = tp.init
+            elif tp.typ.name in ['c_float', 'c_double', 'c_longdouble']:
+                init_value = 0.0
+            else:
+                # integers
+                init_value = 0
         #
-        self.names.add(tp.name)
-        # try:
-        #    value = self.initialize(tp.typ, tp.init)
-        # except (TypeError, ValueError, SyntaxError, NameError), detail:
-        #    log.error("Could not init %s %s %s"% (tp.name, tp.init, detail))
-        #    import code
-        #    code.interact(local=locals())
-        #    return
-        # import code
-        # code.interact(local=locals())
+        # print it out
+        print("%s = %s # Variable %s" % (tp.name, init_value, self.type_name(tp.typ, False)), file=self.stream)
+        #
+        self.names.append(tp.name)
+        return
 
     _enumvalues = 0
 
@@ -434,7 +437,7 @@ class Generator(object):
         # FIXME should be in parser
         value = int(tp.value)
         print("%s = %d" % (tp.name, value), file=self.stream)
-        self.names.add(tp.name)
+        self.names.append(tp.name)
         self._enumvalues += 1
         return
 
@@ -461,7 +464,7 @@ class Generator(object):
             self._generate(item)
         if tp.name:
             print("%s = ctypes.c_int # enum" % tp.name, file=self.stream)
-            self.names.add(tp.name)
+            self.names.append(tp.name)
         self._enumtypes += 1
         return
 
@@ -547,7 +550,7 @@ class Generator(object):
         # special empty struct
         if inline and not head.struct.members:
             print("    pass\n", file=self.stream)
-        self.names.add(head.struct.name)
+        self.names.append(head.struct.name)
         log.debug('Head finished for %s', head.name)
 
     def StructureBody(self, body, inline=False):
@@ -788,7 +791,7 @@ class Generator(object):
                 ret=typeString(func.returns),
             ), file=self.stream)
 
-        self.names.add(func.name)
+        self.names.append(func.name)
         self._functiontypes += 1
         return
 
