@@ -4,6 +4,7 @@ import sys
 import unittest
 
 from test.util import ClangTest
+import ctypeslib
 
 
 def run(args):
@@ -23,6 +24,7 @@ clang2py_path = clang2py_path.strip()
 
 def clang2py(args):
     return run([sys.executable, clang2py_path] + args)
+    # return run([clang2py_path] + args)
 
 
 
@@ -71,6 +73,8 @@ class ArgumentInclude(ClangTest):
     def test_include_with(self):
         """run clang2py -i test/data/test-includes.h"""
         p, output, stderr = clang2py(['-i', 'test/data/test-includes.h'])
+        # print(output)
+        # print(stderr)
         self.assertEqual(0, p.returncode)
         # struct_name are defined in another include file
         self.assertIn("struct_Name", output)
@@ -98,18 +102,6 @@ class ArgumentHelper(ClangTest):
         self.assertIn("optional arguments", output)
 
 
-class ArgumentVersion(ClangTest):
-
-    def test_version(self):
-        """run clang2py -V"""
-        p, output, stderr = clang2py(['-V', 'XXXXX'])
-        self.assertEqual(0, p.returncode)
-        if sys.version_info[0] < 3:
-            self.assertIn("clang2py version", stderr)
-        else:
-            self.assertIn("clang2py version", output)
-
-
 class ArgumentTypeKind(ClangTest):
 
     @unittest.skip('find a good test for aliases')
@@ -184,13 +176,105 @@ class ArgumentTypeKind(ClangTest):
         self.assertNotIn("union__complex3_0_1_1(", output)
 
 
-class ArgumentComments(ClangTest):
+class ArgumentVersion(ClangTest):
 
-    @unittest.skip('find a good test for function')
-    def test_comment(self):
-        """run clang2py -c test/data/test-records-complex.c"""
-        p, output, stderr = clang2py(['-c', 'test/data/test-records-complex.c'])
+    def test_version(self):
+        """run clang2py --version"""
+        p, output, stderr = clang2py(['--version'])
         self.assertEqual(0, p.returncode)
+        self.assertIn(str(ctypeslib.__version__), output)
+        self.assertIn("libclang", output)
+
+    def test_version(self):
+        """run clang2py -V"""
+        p, output, stderr = clang2py(['-V', 'XXXXX'])
+        self.assertEqual(0, p.returncode)
+        if sys.version_info[0] < 3:
+            self.assertIn("clang2py version", stderr)
+        else:
+            self.assertIn("clang2py version", output)
+
+
+class ArgumentVerbose(ClangTest):
+
+    def test_verbose(self):
+        """run clang2py --verbose test/data/test-records.c"""
+        p, output, stderr = clang2py(['--verbose', 'test/data/test-records.c'])
+        self.assertEqual(0, p.returncode)
+        self.assertNotIn("DEBUG:", stderr)
+        self.assertNotIn("DEBUG:", output)
+        self.assertIn("# Total symbols:", stderr)
+
+    def test_debug(self):
+        """run clang2py --verbose test/data/test-records.c"""
+        p, output, stderr = clang2py(['--verbose', 'test/data/test-records.c', '--debug'])
+        self.assertEqual(0, p.returncode)
+        self.assertIn("DEBUG:", stderr)
+        self.assertNotIn("DEBUG:", output)
+        self.assertIn("# Total symbols:", stderr)
+
+
+from io import StringIO
+from unittest.mock import patch
+
+
+class ModuleTesting(ClangTest):
+    def test_version(self):
+        """run clang2py -v"""
+        from ctypeslib import clang2py
+        with patch('sys.stdout', new=StringIO()) as fake_out:
+            with self.assertRaises(SystemExit):
+                clang2py.main(['--version'])
+            self.assertIn(str(ctypeslib.__version__), fake_out.getvalue())
+
+    def test_arg_file(self):
+        """run clang2py test/data/test-basic-types.c"""
+        from ctypeslib import clang2py
+        with patch('sys.stdout', new=StringIO()) as fake_out:
+            clang2py.main(['test/data/test-basic-types.c'])
+            self.assertIn("_int = ctypes.c_int", fake_out.getvalue())
+
+    def test_arg_input_stdin(self):
+        """run echo | clang2py - """
+        from ctypeslib import clang2py
+        with patch('sys.stdin', StringIO('int i = 0;')) as stdin, patch('sys.stdout', new=StringIO()) as fake_out:
+            clang2py.main(['-'])
+            self.assertIn("__all__ =", fake_out.getvalue())
+            self.assertIn("# TARGET arch is: []", fake_out.getvalue())
+
+    def test_arg_debug(self):
+        """run clang2py --debug test/data/test-basic-types.c"""
+        from ctypeslib import clang2py
+        with patch('sys.stdout', new=StringIO()) as fake_out, patch('sys.stderr', new=StringIO()) as fake_err:
+            clang2py.main(['--debug', 'test/data/test-basic-types.c'])
+            self.assertIn("_int = ctypes.c_int", fake_out.getvalue())
+            self.assertIn("DEBUG:clangparser:ARCH sizes:", fake_err.getvalue())
+            self.assertNotIn("ERROR", fake_err.getvalue())
+
+    def test_arg_target(self):
+        """run clang2py --target x86_64-Linux test/data/test-basic-types.c """
+        from ctypeslib import clang2py
+        with patch('sys.stdout', new=StringIO()) as fake_out:
+            clang2py.main(['--target', 'x86_64-Linux', 'test/data/test-basic-types.c'])
+            self.assertIn("# TARGET arch is: ['-target', 'x86_64-Linux']", fake_out.getvalue())
+            self.assertIn("_int = ctypes.c_int", fake_out.getvalue())
+            self.assertIn("_long = ctypes.c_int64", fake_out.getvalue())
+
+            clang2py.main(['--target', 'i586-Linux', 'test/data/test-basic-types.c'])
+            self.assertIn("# TARGET arch is: ['-target', 'i586-Linux']", fake_out.getvalue())
+            self.assertIn("_int = ctypes.c_int", fake_out.getvalue())
+            self.assertIn("_long = ctypes.c_int32", fake_out.getvalue())
+
+    # TODO
+    @unittest.skip
+    def test_arg_clang_args(self):
+        """run clang2py test/data/test-basic-types.c --clang-args="-DDEBUG=2" """
+        from ctypeslib import clang2py
+        with patch('sys.stdin', StringIO('int i = DEBUG;')) as stdin, patch('sys.stdout', new=StringIO()) as fake_out:
+            clang2py.main(['', '--clang-args="-DDEBUG=2"', '-'])
+            self.assertIn("# TARGET arch is: []", fake_out.getvalue())
+            self.assertIn("i = 2", fake_out.getvalue())
+
 
 
 if __name__ == "__main__":
